@@ -29,7 +29,9 @@ extern "C" {
 #include <ofono/types.h>
 
 struct ofono_modem;
+struct ofono_gprs;
 struct ofono_sim;
+struct ofono_voicecall;
 
 enum ofono_modem_type {
 	OFONO_MODEM_TYPE_HARDWARE = 0,
@@ -82,6 +84,8 @@ void ofono_modem_remove_interface(struct ofono_modem *modem,
 
 const char *ofono_modem_get_path(struct ofono_modem *modem);
 struct ofono_sim *ofono_modem_get_sim(struct ofono_modem *modem);
+struct ofono_gprs *ofono_modem_get_gprs(struct ofono_modem *modem);
+struct ofono_voicecall *ofono_modem_get_voicecall(struct ofono_modem *modem);
 
 void ofono_modem_set_data(struct ofono_modem *modem, void *data);
 void *ofono_modem_get_data(struct ofono_modem *modem);
