@@ -244,6 +244,7 @@ static void lte_atom_remove(struct ofono_atom *atom)
 }
 
 struct ofono_lte *ofono_lte_create(struct ofono_modem *modem,
+					unsigned int vendor,
 					const char *driver, void *data)
 {
 	struct ofono_lte *lte;
@@ -266,7 +267,7 @@ struct ofono_lte *ofono_lte_create(struct ofono_modem *modem,
 		if (g_strcmp0(drv->name, driver))
 			continue;
 
-		if (drv->probe(lte, data) < 0)
+		if (drv->probe(lte, vendor, data) < 0)
 			continue;
 
 		lte->driver = drv;
@@ -342,7 +343,7 @@ void ofono_lte_register(struct ofono_lte *lte)
 	const char *imsi = ofono_sim_get_imsi(sim);
 
 	if (imsi == NULL) {
-		ofono_error("No sim atom required for registering LTE atom.");
+		ofono_error("No sim atom found. It is required for registering LTE atom.");
 		return;
 	}
 
@@ -372,3 +373,8 @@ void *ofono_lte_get_data(const struct ofono_lte *lte)
 {
 	return lte->driver_data;
 }
+
+struct ofono_modem *ofono_lte_get_modem(const struct ofono_lte *lte)
+{
+	return __ofono_atom_get_modem(lte->atom);
+}
